package jalview.gui;

import jalview.jbgui.GUserDefinedColours;
import jalview.datamodel.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import jalview.schemes.*;
import java.io.*;
import java.util.*;


public class UserDefinedColours extends GUserDefinedColours implements ChangeListener
{

  AlignmentPanel ap;
  SequenceGroup seqGroup;
  JButton selectedButton;
  Vector oldColours = new Vector();
  JInternalFrame frame;

  public UserDefinedColours(JInternalFrame f, AlignmentPanel ap, SequenceGroup sg)
  {
    super();
    colorChooser.getSelectionModel().addChangeListener(this);
    this.ap = ap;
    frame = f;
   seqGroup = sg;

    for(int i=0; i<20; i++)
      makeButton(ResidueProperties.aa2Triplet.get( ResidueProperties.aa[i] )+"", ResidueProperties.aa[i]);

    makeButton("B","B");
    makeButton("Z","Z");
    makeButton("X","X");
    makeButton("'.','-',' '", "-");

  }

 public void stateChanged(ChangeEvent evt)
 {
   if(selectedButton!=null)
     selectedButton.setBackground( colorChooser.getColor() );
 }

  public void colourButtonPressed(MouseEvent e)
  {
     selectedButton = (JButton)e.getSource();
     colorChooser.setColor(selectedButton.getBackground());
  }

  void makeButton(String label, String aa)
  {
    final JButton button = new JButton();
    Color col = Color.white;
    ColourSchemeI cs = null;
    if(seqGroup!=null)
      cs = seqGroup.cs;
    else
      cs = ap.av.getGlobalColourScheme();

    if(cs!=null)
      col = cs.findColour( aa, -1, null);

    button.setBackground(  col );
    oldColours.addElement( col );
    button.setText( label );
    button.setForeground( col.darker().darker().darker() );
    button.setFont(new java.awt.Font("Verdana", 1, 10));
    button.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        colourButtonPressed(e);
      }
    });

    buttonPanel.add(button);
  }


  protected void okButton_actionPerformed(ActionEvent e)
  {
    applyButton_actionPerformed(null);
    try{
      frame.setClosed(true);
    }catch(Exception ex){}
  }

  protected void applyButton_actionPerformed(ActionEvent e)
  {
    UserColourScheme ucs = new UserColourScheme();
    Color [] newColours = new Color[24];
    for(int i=0; i<24; i++)
    {
          JButton button = (JButton)buttonPanel.getComponent(i);
          newColours[i] = button.getBackground();
    }

    ucs.setThreshold(0);
    ucs.setColourScheme( newColours );

    if(seqGroup!=null)
      seqGroup.cs = ucs;
    else
      ap.av.setGlobalColourScheme(ucs);

    ap.RefreshPanels();
  }

  protected void loadbutton_actionPerformed(ActionEvent e)
  {
    JFileChooser chooser = new JFileChooser(jalview.bin.Cache.LAST_DIRECTORY);
    chooser.setDialogTitle("Load colour scheme");
    chooser.setToolTipText("Load");
    int value = chooser.showOpenDialog(this);
    if (value == JFileChooser.APPROVE_OPTION)
    {
      String choice = chooser.getSelectedFile().getPath();
      jalview.bin.Cache.LAST_DIRECTORY = choice;

      try{
        BufferedReader in = new BufferedReader(new FileReader(choice));
        for(int i=0; i<24; i++)
        {
          JButton button = (JButton)buttonPanel.getComponent(i);
          Color c = new Color(Integer.parseInt(in.readLine()));
          button.setBackground(c);
        }

      }
      catch(Exception ex)
      {}
    }

  }

  protected void savebutton_actionPerformed(ActionEvent e)
  {
    JFileChooser chooser = new JFileChooser(jalview.bin.Cache.LAST_DIRECTORY);
    chooser.setDialogTitle("Save colour scheme");
    chooser.setToolTipText("Save");
    int value = chooser.showSaveDialog(this);
    if (value == JFileChooser.APPROVE_OPTION)
    {
      String choice = chooser.getSelectedFile().getPath();
      jalview.bin.Cache.LAST_DIRECTORY = choice;

      try{
        PrintWriter out = new PrintWriter(new FileWriter(choice));
        for(int i=0; i<24; i++)
        {
          JButton button = (JButton)buttonPanel.getComponent(i);
          out.println(button.getBackground().getRGB() + "");
        }

        out.close();
      }
      catch(Exception ex)
      {ex.printStackTrace();}
    }

  }

  protected void cancelButton_actionPerformed(ActionEvent e)
  {
    UserColourScheme ucs = new UserColourScheme();
    Color [] newColours = new Color[24];
    for(int i=0; i<24; i++)
    {
          newColours[i] = (Color)oldColours.elementAt(i);
          buttonPanel.getComponent(i).setBackground(newColours[i]);
    }

    ucs.setColourScheme( newColours );
    if (seqGroup != null)
      seqGroup.cs = ucs;
    else
      ap.av.setGlobalColourScheme(ucs);

    ap.RefreshPanels();

  }


}
